<?php

namespace Drupal\config_enforce\Form;

use Drupal\config_enforce\EnforcedConfig;
use Drupal\config_enforce\Form\FormHelperTrait;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Attached Config Enforce behaviour for config forms.
 */
abstract class AbstractEnforceForm extends ConfigFormBase {

  use FormHelperTrait;

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->setSharedFormProperties($form, $form_state);
  }

  /**
   * Make our form float above the config form to which it's attached.
   */
  protected function makeFormFloat() {
    $this->form()['#prefix'] = "<div class='config-enforce-form'>";
    $this->form()['#suffix'] = "</div>";
    $this->form()['#attached']['library'][] = 'config_enforce/config-enforce';
    return $this;
  }

  /**
   * Add a details section with information about the config form.
   */
  protected function addConfigFormInfo() {
    $form_id = $this->getContext('form_id');

    $this->form()['config_enforce'] = [
      '#type' => 'details',
      '#tree' =>TRUE,
      '#title' => $this->t('Config form: %id', ['%id' => $form_id]),
      '#open' => !$this->allConfigsAreEnforced(),
      '#attributes' => [
        'class' => $this->getFormInfoClasses(),
      ],
    ];

    $this->addEnforcedConfigItems();
    return $this;
  }

  /**
   * Determine whether all config objects in this form are enforced.
   */
  protected function allConfigsAreEnforced() {
    foreach ($this->getFormConfigs() as $config_name) {
      if (!EnforcedConfig::isEnforced($config_name)) return FALSE;
    }
    return TRUE;
  }

  /**
   * Return form info classes to allow for better styling.
   */
  protected function getFormInfoClasses() {
    if (!$this->allConfigsAreEnforced()) return;
    $enforcement_levels = [];
    foreach ($this->getFormConfigs() as $config_name) {
      $enforcement_levels[$config_name] = EnforcedConfig::getEnforcementLevel($config_name);
    }
    return [
      'enforced',
       'level-' . min($enforcement_levels),
    ];
  }

  /**
   * Add enforcement settings for each config object managed by this form.
   */
  protected function addEnforcedConfigItems(){
    foreach ($this->getFormConfigs() as $config_name) {
      $this->addEnforcedConfigItem($config_name);
    }
    return $this;
  }

  /**
   * Add enforcement settings for a single config object managed by this form.
   */
  protected function addEnforcedConfigItem($config_name) {
    $this->setCurrentConfig($config_name);

    $this->form()['config_enforce'][$config_name] = [
      '#type' => 'details',
      '#title' => $this->t('Config object: @config', ['@config' => $config_name]),
      '#open' => !EnforcedConfig::isEnforced($config_name),
      '#attributes' => [
        'class' => $this->getRowClasses(),
      ],
    ];

    $config_enforce_enabled = EnforcedConfig::isEnforced($config_name);
    $this->form()['config_enforce'][$config_name]['config_enforce_enabled'] = [
      '#type' => 'item',
      '#title' => $this->t('Config enforce'),
      '#markup' => $config_enforce_enabled ? 'Enabled' : 'Disabled',
    ];

    if ($config_enforce_enabled) {

      $this->form()['config_enforce'][$config_name]['target_module'] = [
        '#type' => 'item',
        '#title' => $this->t('Module'),
        '#markup' => EnforcedConfig::getTargetModule($config_name),
      ];

      $this->form()['config_enforce'][$config_name]['config_directory'] = [
        '#type' => 'item',
        '#title' => $this->t('Config directory'),
        '#markup' => EnforcedConfig::getConfigDirectory($config_name),
      ];

      $this->form()['config_enforce'][$config_name]['config_enforce_path'] = [
        '#type' => 'item',
        '#title' => $this->t('Config file path'),
        '#markup' => EnforcedConfig::getConfigFilePath($config_name),
      ];

      $this->form()['config_enforce'][$config_name]['enforcement_level'] = [
        '#type' => 'item',
        '#title' => $this->t('Enforcement level'),
        '#markup' => EnforcedConfig::getEnforcementLevelLabel($config_name),
      ];

    }
    return $this;
  }

  /**
   * Return an item passed into the form from the config form to which we are attached.
   */
  protected function getContext($key = '') {
    $context = $this->formState()->getBuildInfo()['args'];
    return empty($key) ? $context : $context[$key];
  }

  /**
   * Return a list of configs embedded in the current form object.
   */
  protected function getFormConfigs() {
    return $this->getContext('configs');
  }

}
